@@ -1,7 +1,13 @@
+from __future__ import annotations
+
+import logging
 import os
-from pathlib import Path
+from enum import Enum
 from urllib.parse import urlparse
-from typing import Optional, Union
+from typing import Optional, Union, Callable
+
+from .context import Context
+from .constants import ContextDictType
 
 
 class MissingApiKeyException(Exception):
@@ -31,31 +37,23 @@ class InvalidApiUrlException(Exception):
         super().__init__(f"Invalid API URL found: {url}")
 
 
-class InvalidGrpcUrlException(Exception):
-    """
-    Raised when an invalid gRPC URL is given
-    """
-
-    def __init__(self, url: str) -> None:
-        super().__init__(f"Invalid gRPC URL found: {url}")
-
-
 VALID_DATASOURCES = ("LOCAL_ONLY", "ALL")
 VALID_ON_NO_DEFAULT = ("RAISE", "RETURN_NONE")
 VALID_ON_CONNECTION_FAILURE = ("RETURN", "RAISE")
 
 
 class Options:
+    class ContextUploadMode(Enum):
+        NONE = 1
+        SHAPE_ONLY = 2
+        PERIODIC_EXAMPLE = 3
+
     def __init__(
         self,
         api_key: Optional[str] = None,
-        prefab_api_url: Optional[str] = None,
-        prefab_grpc_url: Optional[str] = None,
+        prefab_api_urls: Optional[list[str]] = None,
+        prefab_telemetry_url: Optional[str] = None,
         prefab_datasources: Optional[str] = None,
-        logdev: str = "STDIO",
-        log_prefix: Optional[str] = None,
-        log_boundary: Optional[str] = None,
-        namespace: str = "",
         connection_timeout_seconds: int = 10,
         prefab_config_override_dir: Optional[str] = os.environ.get("HOME"),
         prefab_config_classpath_dir: str = ".",
@@ -64,25 +62,28 @@ class Options:
         on_no_default: str = "RAISE",
         on_connection_failure: str = "RETURN",
         x_use_local_cache: bool = False,
+        x_datafile: Optional[str] = None,
         collect_logs: bool = True,
         collect_max_paths: int = 1000,
         collect_max_shapes: int = 10_000,
-        collect_sync_interval: Optional[int] = None,
+        collect_sync_interval: Optional[int] = 30,
+        collect_evaluation_summaries: bool = True,
+        context_upload_mode: ContextUploadMode = ContextUploadMode.PERIODIC_EXAMPLE,
+        bootstrap_loglevel: Optional[int] = None,
+        global_context: Optional[ContextDictType | Context] = None,
+        on_ready_callback: Optional[Callable[[], None]] = None,
     ) -> None:
         self.prefab_datasources = Options.__validate_datasource(prefab_datasources)
+        self.datafile = x_datafile
         self.__set_api_key(api_key or os.environ.get("PREFAB_API_KEY"))
         self.__set_api_url(
-            prefab_api_url
-            or os.environ.get("PREFAB_API_URL")
-            or "https://api.prefab.cloud"
+            prefab_api_urls
+            or self.api_urls_from_env()
+            or ["https://belt.prefab.cloud", "https://suspenders.prefab.cloud"]
         )
-        self.__set_grpc_url(
-            prefab_grpc_url or os.getenv("PREFAB_GRPC_URL") or "grpc.prefab.cloud:443"
+        self.telemetry_url = self.validate_url(
+            prefab_telemetry_url or "https://telemetry.prefab.cloud"
         )
-        self.logdev = logdev
-        self.log_prefix = log_prefix
-        self.log_boundary = str(Path(log_boundary).resolve()) if log_boundary else None
-        self.namespace = namespace
         self.connection_timeout_seconds = connection_timeout_seconds
         self.prefab_config_override_dir = prefab_config_override_dir
         self.prefab_config_classpath_dir = prefab_config_classpath_dir
@@ -91,29 +92,29 @@ class Options:
         self.stats = None
         self.shared_cache = None
         self.use_local_cache = x_use_local_cache
-        self.__set_url_for_api_cdn()
         self.__set_on_no_default(on_no_default)
         self.__set_on_connection_failure(on_connection_failure)
         self.__set_log_collection(collect_logs, collect_max_paths, self.is_local_only())
         self.collect_sync_interval = collect_sync_interval
         self.collect_max_shapes = collect_max_shapes
+        self.context_upload_mode = context_upload_mode
+        self.collect_evaluation_summaries = collect_evaluation_summaries
+        self.bootstrap_loglevel = (
+            os.environ.get("PREFAB_LOG_CLIENT_BOOTSTRAP_LOG_LEVEL")
+            or bootstrap_loglevel
+            or logging.WARNING
+        )
+        self.global_context = Context.normalize_context_arg(global_context)
+        self.on_ready_callback = on_ready_callback
 
     def is_local_only(self) -> bool:
         return self.prefab_datasources == "LOCAL_ONLY"
 
-    def __set_url_for_api_cdn(self) -> None:
-        if self.prefab_datasources == "LOCAL_ONLY":
-            self.url_for_api_cdn = None
-        else:
-            cdn_url_from_env = os.environ.get("PREFAB_CDN_URL")
-            if cdn_url_from_env is not None:
-                self.url_for_api_cdn = cdn_url_from_env
-            elif self.prefab_api_url:
-                self.url_for_api_cdn = (
-                    self.prefab_api_url.replace(".", "-") + ".global.ssl.fastly.net"
-                )
-            else:
-                self.url_for_api_cdn = None  # TODO: should we warn?
+    def has_datafile(self) -> bool:
+        return self.datafile is not None
+
+    def is_loading_from_api(self) -> bool:
+        return not (self.is_local_only() or self.has_datafile())
 
     @staticmethod
     def __validate_datasource(datasource: Optional[str]) -> str:
@@ -128,7 +129,7 @@ class Options:
             return default
 
     def __set_api_key(self, api_key: Optional[str]) -> None:
-        if self.prefab_datasources == "LOCAL_ONLY":
+        if self.is_local_only() or self.has_datafile():
             self.api_key = None
             self.api_key_id = "local"
             return
@@ -141,28 +142,35 @@ class Options:
         self.api_key = api_key
         self.api_key_id = api_key.split("-")[0]
 
-    def __set_api_url(self, api_url: Optional[str]) -> None:
+    def __set_api_url(self, api_url_list: list[str]) -> None:
         if self.prefab_datasources == "LOCAL_ONLY":
-            self.prefab_api_url = None
+            self.prefab_api_urls = None
             return
+        self.prefab_api_urls = self.validate_and_process_urls(api_url_list)
 
-        api_url = str(api_url)
-        parsed_url = urlparse(api_url)
+    @staticmethod
+    def validate_url(api_url: str) -> str:
+        parsed_url = urlparse(str(api_url))
         if parsed_url.scheme in ["http", "https"]:
-            self.prefab_api_url = api_url.rstrip("/")
+            return api_url.rstrip("/")
         else:
             raise InvalidApiUrlException(api_url)
 
-    def __set_grpc_url(self, grpc_url: Optional[str]) -> None:
-        if self.prefab_datasources == "LOCAL_ONLY":
-            self.prefab_grpc_url = None
-            return
+    @staticmethod
+    def api_urls_from_env() -> Optional[list[str]]:
+        envvar = os.environ.get("PREFAB_API_URL")
+        if envvar:
+            return envvar.split(",")
+        return None
 
-        grpc_url = str(grpc_url)
-        if grpc_url.startswith("grpc."):
-            self.prefab_grpc_url = grpc_url
-        else:
-            raise InvalidGrpcUrlException(grpc_url)
+    def validate_and_process_urls(self, api_url_list: list[str]) -> list[str]:
+        valid_urls = []
+        for api_url in api_url_list:
+            try:
+                valid_urls.append(self.validate_url(api_url))
+            except InvalidApiUrlException as e:
+                raise e
+        return valid_urls
 
     @classmethod
     def __construct_prefab_envs(cls, envs_from_input: list[str]) -> list[str]:
